package Echiquier.Vue;

import Echiquier.Echiquier;
import Echiquier.Modele.Case;
import Echiquier.Modele.ColorJoueur;
import Echiquier.Modele.Piece.*;

import java.util.Scanner;

public class IHM {
    private Echiquier echiquier;
    private Case[][] table;

    public IHM(Echiquier echiquier){
        this.echiquier = echiquier;
        this.table = this.echiquier.getTable();
        affichage();

        menu();
    }

    private void affichage(){
        //afficher la numero de colonne du haut
        afficheCol();

        //afficher la table de l'echiquier
        for(int lig=0; lig < Echiquier.MAXLIG; lig++){
            //afficher la separation de ligne
            afficheLigSep();

            //afficher la numero de ligne à gauche
            System.out.print(Echiquier.MAXLIG - lig);
            for(int col=0; col < Echiquier.MAXCOL; col++){
                //afficher la separation de colonne
                System.out.print("|");

                //afficher de la symbole de piece
                if(table[lig][col].estEuPiece()){
                    Piece piece = table[lig][col].getPiece();
                    if(piece.getClass() == Pion.class ){
                        System.out.print("P");
                    }
                    if(piece.getClass() == Cavalier.class){
                        System.out.print("C");
                    }
                    if(piece.getClass() == Dame.class){
                        System.out.print("D");
                    }
                    if(piece.getClass() == Fou.class){
                        System.out.print("F");
                    }
                    if(piece.getClass() == Roi.class){
                        System.out.print("R");
                    }
                    if(piece.getClass() == Tour.class){
                        System.out.print("T");
                    }

                    //affichier la color du joueur
                    if(piece.getColorJoueur() == ColorJoueur.NOIR){
                        System.out.print("N");
                    }
                    else {
                        System.out.print("B");
                    }
                }
                //afficher la vide case
                else{
                    System.out.print("  ");
                }
            }
            //afficher la dernier separation de colonne
            System.out.print("|");
            //affichier la ligne à droit
            System.out.println(Echiquier.MAXLIG - lig);
        }

        //afficher la dernier separation
        afficheLigSep();

        //afficher la numero de colonne du base
        afficheCol();
    }

    //afficher la separation de ligne
    private void afficheLigSep() {
        System.out.print("  ");
        for(int tmplig = 0; tmplig < Echiquier.MAXCOL; tmplig++) {
            System.out.print("---");
        }
        System.out.println();
    }

    //afficher la numero de colonne
    private void afficheCol() {
        for(int tmpcol = 0; tmpcol < Echiquier.MAXCOL; tmpcol++) {
            char numero = (char) ('A' + tmpcol);
            System.out.print("  " + numero);
        }
        System.out.println();
    }

    private void menu(){
        Case[] caseDepart;
        while(true){
            //recupperer le case que le joueur veut bouger
            try {
                caseDepart = saisirString("Indique le mouvement (B2-B4 ou bien b2-4B) par exemple :");

                if(null != caseDepart) {
                    echiquier.deplace(caseDepart);
                    affichage();
                }
            }catch (Exception e){
                System.out.println(e.getMessage());
            }
        }
    }

    private Case[] saisirString(String message){
        Case[] cases = new Case[2];

        System.out.println(message);
        String valeur = "";

        Case unCase = null;
        try{
            //Saisire les valeurs
            Scanner sc = new Scanner(System.in);
            valeur = sc.nextLine();

            String[] divise = valeur.split("[:,-]");

            //Si l'entre n'est pas deux position
            if(divise.length != 2){
                throw new Exception("Saisir deux position disponible diviser par \":\" \"-\" ");
            }

            //Traiter les mauvais positions
            int cpt = 0;
            for(String tmp : divise) {
                char[] chars = tmp.toUpperCase().toCharArray();
                if(chars.length != 2){
                    throw new Exception("La position incorrect !");
                }

                //traitement saisir deux colonne ou deux ligne
                if (chars[0] >= 'A' && chars[0] <= 'H' && chars[1] >= 'A' && chars[1] <= 'H'
                        || chars[0] >= '1' && chars[0] <= '8' && chars[1] >= '1' && chars[1] <= '8') {
                    throw new Exception("La position incorrect ! \n vous avez saisi deux colonne ou deux ligne. ");
                }

                //traiter la position exist
                if(  ! (chars[0] >= 'A' && chars[0] <= 'H' || chars[1] >= 'A' && chars[1] <= 'H') ||
                        !(chars[0] >= '1' && chars[0] <= '8' || chars[1] >= '1' && chars[1] <= '8'))
                    throw new Exception("La position incoorect ! \n La position n'exit pas.");

                //traitement saisir la ligne et la colonne s'il est invert.
                if (chars[0] >= 'A' && chars[0] <= 'H' ) {
                    cases[cpt] = new Case(Echiquier.MAXLIG - (chars[1] - '1') - 1, chars[0] - 'A');
                } else {
                    cases[cpt] = new Case(Echiquier.MAXLIG - (chars[0] - '1') - 1,chars[1] - 'A');
                }
                cpt++;
            }
        }catch (Exception e){
            System.out.println(e.getMessage());
            return null;
        }

        return cases;
    }
}
